<?php

namespace Drupal\speakeasy\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\Config;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\Html;

/**
 * Provides a 'SpeakEasy' block.
 *
 * @Block(
 *   id = "speakeasy_block",
 *   admin_label = @Translation("SpeakEasy Block"),
 * )
 */
class SpeakEasyBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new SpeakEasyBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\Config $config
   *   The configuration for this module.
   * @param \Drupal\node\NodeInterface|null $node
   *   The node whose page is being viewed, if any.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected Config $config,
    protected ?NodeInterface $node,
    protected AccountInterface $currentUser,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')->get('speakeasy.settings'),
      $container->get('current_route_match')->getParameter('node'),
      $container->get('current_user'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'output_style' => 'default',
      'button_text' => $this->t('Listen to this page'),
      'stop_button_text' => $this->t('Stop'),
      'voice_name' => '',
      'rate' => 1,
      'pitch' => 1,
      'show_voice_select' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    if (!$this->node) {
      // If no node is being displayed, do not render the block.
      return [];
    }

    // Collect all text content from the node fields.
    $content = '';

    // Loop through all fields on the node and collect text content.
    foreach ($this->node->getFieldDefinitions() as $field_name => $field_definition) {
      if ($this->node->hasField($field_name)) {
        $field = $this->node->get($field_name);

        if (!$field->isEmpty() && in_array($field_definition->getType(), ['string', 'string_long', 'text', 'text_long', 'text_with_summary', 'text_plain', 'email', 'telephone'])) {
          // For multi-valued fields, concatenate all values.
          foreach ($field as $item) {
            $text = $item->value;
            // Strip HTML tags and decode HTML entities to ensure plain text.
            $text = Html::decodeEntities(strip_tags($text));
            $content .= (strlen($content) > 0 ? ' ' : '') . $text;
          }
        }

        // Check if this field references Paragraphs.
        if ($field_definition->getType() == 'entity_reference_revisions' && $field->getSetting('target_type') == 'paragraph') {
          /** @var \Drupal\paragraphs\ParagraphInterface[] $paragraphs */
          $paragraphs = $field->referencedEntities();
          foreach ($paragraphs as $paragraph) {
            foreach ($paragraph->getFieldDefinitions() as $para_field_name => $para_field_definition) {
              if ($paragraph->hasField($para_field_name)) {
                $para_field = $paragraph->get($para_field_name);
                if (!$para_field->isEmpty() && in_array($para_field_definition->getType(), ['text', 'text_long', 'text_with_summary'])) {
                  // For multi-valued fields in paragraphs.
                  foreach ($para_field as $item) {
                    $para_text = $item->value;
                    $para_text = Html::decodeEntities(strip_tags($para_text));
                    $content .= (strlen($content) > 0 ? ' ' : '') . $para_text;
                  }
                }
              }
            }
          }
        }
      }
    }

    // Pass the combined plain text content to drupalSettings.
    $voice_name = $this->configuration['voice_name'] ?: $this->config->get('default_voice_name');
    $rate = $this->configuration['rate'] ?: $this->config->get('default_rate') ?: 1;
    $pitch = $this->configuration['pitch'] ?: $this->config->get('default_pitch') ?: 1;
    $output_style = $this->configuration['output_style'] ?? 'default';
    $show_voice_select = $this->configuration['show_voice_select'] ?? TRUE;

    // Build the block.
    $build = [
      '#type' => 'container',
      '#attributes' => ['class' => ['speakeasy-container']],
      '#attached' => [
        'library' => ['speakeasy/speakeasy'],
        'drupalSettings' => [
          'speakeasy' => [
            'content' => $content, // Pass combined plain text content to JS.
            'voiceName' => $voice_name,
            'rate' => $rate,
            'pitch' => $pitch,
            'outputStyle' => $output_style,
          ],
        ],
      ],
    ];

    // Add cache contexts and tags to ensure the block is unique per node.
    $build['#cache']['contexts'][] = 'route';
    if ($nid = $this->node->id()) {
      $build['#cache']['tags'][] = "node:$nid";
    }

    switch ($output_style) {
      case 'simple_link':
        // Example for 'simple_link' output style.
        $build['simple_link'] = [
          '#type' => 'link',
          '#title' => $this->t('Listen'),
          '#url' => Url::fromRoute('<none>'),
          '#attributes' => [
            'id' => 'speakeasy-link',
            'class' => ['speakeasy-link'],
            'href' => '#',
            // Indicate it's a button-like link.
            'role' => 'button',
            // Indicate toggle state.
            'aria-pressed' => 'false',
          ],
        ];
        break;

      case 'media_player':
        $build['media_player'] = [
          '#theme' => 'speakeasy_media_player',
          '#attached' => ['library' => ['speakeasy/speakeasy']],
        ];
        break;

      default:
        $build['voice_select'] = [
          '#type' => 'html_tag',
          '#tag' => 'select',
          '#attributes' => [
            'id' => 'speakeasy-voice-select',
            'class' => ['speakeasy-voice-select'],
          ],
        ];
        if (!$show_voice_select) {
          $build['voice_select']['#attributes']['style'] = 'display:none';
        }

        $build['listen_button'] = [
          '#type' => 'html_tag',
          '#tag' => 'button',
          '#value' => $this->configuration['button_text'],
          '#attributes' => [
            'id' => 'speakeasy-tts-button',
            'class' => ['speakeasy-button'],
            'aria-label' => $this->t('Listen to the content on this page'),
          ],
        ];
        

        $build['stop_button'] = [
          '#type' => 'html_tag',
          '#tag' => 'button',
          '#value' => $this->configuration['stop_button_text'],
          '#attributes' => ['id' => 'speakeasy-stop-button', 'class' => ['speakeasy-button']],
        ];
        break;
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    // Add a link to the settings page.
    if ($this->currentUser->hasPermission('access speakeasy settings')) {
      $form['speakeasy_settings_link'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<p>To configure default settings, visit the <a href="@url" target="_blank">SpeakEasy settings page</a>.</p>', [
          '@url' => Url::fromRoute('speakeasy.settings')->toString(),
        ]),
        '#weight' => -10,
      ];
    }

    $form['output_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Output Style'),
      '#description' => $this->t('Select the display style for the SpeakEasy block.'),
      '#options' => [
        'simple_link' => $this->t('Simple Link'),
        'media_player' => $this->t('Media Player'),
        'default' => $this->t('Default'),
      ],
      '#default_value' => $this->configuration['output_style'],
    ];

    $form['button_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Listen Button Text'),
      '#description' => $this->t('Text displayed on the listen button.'),
      '#default_value' => $this->configuration['button_text'],
      '#states' => [
        'visible' => [
          ':input[name="settings[output_style]"]' => ['value' => 'default'],
        ],
      ],
    ];

    $form['stop_button_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Stop Button Text'),
      '#description' => $this->t('Text displayed on the stop button.'),
      '#default_value' => $this->configuration['stop_button_text'],
      '#states' => [
        'visible' => [
          ':input[name="settings[output_style]"]' => ['value' => 'default'],
        ],
      ],
    ];

    $form['voice_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Voice Name'),
      '#description' => $this->t('Name of the default voice to use. Leave blank for browser default.'),
      '#default_value' => $this->configuration['voice_name'],
    ];

    $form['rate'] = [
      '#type' => 'number',
      '#title' => $this->t('Rate'),
      '#description' => $this->t('Speech rate (0.1 to 10). Default is 1.'),
      '#default_value' => $this->configuration['rate'],
      '#step' => 0.1,
      '#min' => 0.1,
      '#max' => 10,
    ];

    $form['pitch'] = [
      '#type' => 'number',
      '#title' => $this->t('Pitch'),
      '#description' => $this->t('Speech pitch (0 to 2). Default is 1.'),
      '#default_value' => $this->configuration['pitch'],
      '#step' => 0.1,
      '#min' => 0,
      '#max' => 2,
    ];

    $form['show_voice_select'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Voice Selection Dropdown'),
      '#description' => $this->t('Allow users to select from available voices.'),
      '#default_value' => $this->configuration['show_voice_select'],
      '#states' => [
        'visible' => [
          ':input[name="settings[output_style]"]' => ['value' => 'default'],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    foreach (array_keys($this->defaultConfiguration()) as $config_field) {
      $this->configuration[$config_field] = $form_state->getValue($config_field);
    }
  }

}
